#include "shared.h"
#include <cmath>
#include "analysisFunction.h"


class depth:public general{
public:
  int doDepth;
  int maxDepth;
  int doCounts;
  //none optional stuff
  FILE *outfile;
  FILE *outfileG;
  depth(const char *outfiles,argStruct *arguments,int inputtype);
  ~depth();
  void getOptions(argStruct *arguments);
  void run(funkyPars  *pars);
  void clean(funkyPars *pars);
  void print(funkyPars *pars);
  void printArg(FILE *argFile);

  int nInd;
  size_t **depthCount;
  size_t *globCount;
};


void depth::printArg(FILE *argFile){
  fprintf(argFile,"%s:\n",__FILE__);
  fprintf(argFile,"-doDepth\t%d\n",doDepth);
  fprintf(argFile,"-maxDepth\t%d\n",maxDepth);
  fprintf(argFile,"\n");
}

void depth::getOptions(argStruct *arguments){
  //default
  doDepth=0;
  maxDepth=100;
  doCounts=0;
  //from command line
  doDepth=angsd::getArg("-doDepth",doDepth,arguments);
  if(doDepth==-999){
    doDepth=0;
    printArg(stdout);
    exit(0);
  }
  if(doDepth==0)
    return;

  maxDepth=angsd::getArg("-maxDepth",maxDepth,arguments);
  doCounts=angsd::getArg("-doCounts",doCounts,arguments);

  if(doCounts==0){
    fprintf(stderr,"You must use doCounts\n");
    exit(0);
  }

  printArg(arguments->argumentFile);

}

depth::depth(const char *outfiles,argStruct *arguments,int inputtype){

  getOptions(arguments);

  if(doDepth==0)
    return;

  fprintf(stderr,"Estimating DEPTH: %s\n",outfiles);
  //make output files
  const char* postfix;
  postfix=".depthSample";
  outfile = openFile(outfiles,postfix);
  postfix=".depthGlobal";
  outfileG = openFile(outfiles,postfix);

  // Define depthCount // line
  nInd=arguments->nInd;
  depthCount=new size_t *[nInd];
  for(int i=0;i<nInd;i++)
    depthCount[i]=new size_t[maxDepth+1];
  for(int i=0;i<nInd;i++)
    for(int j=0;j<maxDepth+1;j++)
      depthCount[i][j]=0;
  
  globCount = new size_t[maxDepth+1];
  memset(globCount,0,sizeof(size_t)*(maxDepth+1));
}


depth::~depth(){
  //  fprintf(stderr,"[%s] calling destructor\n",__FUNCTION__);
  //  fflush(stderr);
  if(doDepth==0)
    return;
  for(int i=0;i<nInd;i++){
    for(int j=0;j<maxDepth+1;j++){
      fprintf(outfile,"%lu \t",depthCount[i][j]);
    }
    fprintf(outfile,"\n");
  }
  //thorfinn
  for(int j=0;j<maxDepth+1;j++)
    fprintf(outfileG,"%lu \t",globCount[j]);
  fprintf(outfileG,"\n");


  //clean depthCount
  for(int i=0;i<nInd;i++)
    delete[]  depthCount[i];
  delete[] depthCount; 

  fclose(outfile);
  fclose(outfileG);
}


void depth::clean(funkyPars *pars){
  if(doDepth==0)
    return;

}

void depth::print(funkyPars *pars){
  if(doDepth==0)
    return;
  if(pars->counts==NULL){
    fprintf(stderr,"[%s] counts are NULL\n",__FUNCTION__);
    exit(0);
  }

 
  for(int s=0;s<pars->numSites;s++) {
    if(pars->keepSites[s]==0)
      continue; 
    for(int i=0;i<pars->nInd;i++){
      int sum=0;
      for(int a=0;a<4;a++)
	sum+=pars->counts[s][i*4+a];
      if(sum>maxDepth){
	sum=maxDepth;
      }
      depthCount[i][sum]++;	
    }
  }
  //thorfinn below
 
  for(int s=0;s<pars->numSites;s++) {
    if(pars->keepSites[s]==0)
      continue; 
    int sum=0;
    for(int i=0;i<4*pars->nInd;i++){
      sum+=pars->counts[s][i];
      if(sum>maxDepth){
	sum=maxDepth;
      }
      globCount[sum]++;	
    }
  } 
  
}


void depth::run(funkyPars *pars){
  if(doDepth==0)
    return;



}


